const { GObject, Gtk, Gio } = imports.gi;
const GTK_MAJOR_VERSION = Gtk.get_major_version();
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Version = `${Me.metadata.version}.${Me.metadata.minor}.${Me.metadata.revision}`;

function init() {}

const ServerPrefsWidget = new GObject.registerClass(
    class ExtensionHomeServerPrefsWidget extends Gtk.Frame {
        _setMargin(widget, numPixels) {
            if (GTK_MAJOR_VERSION >= 4) {
                widget.set_margin_start(numPixels);
                widget.set_margin_end(numPixels);
            } else {
                widget.set_margin_left(numPixels);
                widget.set_margin_right(numPixels);
            }
            widget.set_margin_top(numPixels);
            widget.set_margin_bottom(numPixels);
        }

        _addGrid(spacing = 10, margin = 10) {
            this._grid = new Gtk.Grid();
            this._grid.set_column_homogeneous(true);
            this._grid.set_column_spacing(spacing);
            this._grid.set_row_spacing(spacing);
            this._setMargin(this, margin);
            this._setMargin(this._grid, margin);
            if (GTK_MAJOR_VERSION >= 4)
                this.set_child(this._grid);
            else
                this.add(this._grid);
        }

        _addLabel(text, left, top, width = 1) {
            let _label = new Gtk.Label({
                label: text,
                halign: Gtk.Align.START,
                use_markup: true,
                visible: true
            });
            this._grid.attach(_label, left, top, width, 1);
        }

        _addEntry(binding, left, top, width = 1) {
            let _entry = Gtk.Entry.new();
            this._grid.attach(_entry, left, top, width, 1);
            this._settings.bind(binding, _entry, 'text', Gio.SettingsBindFlags.DEFAULT);
        }

        _addSwitch(binding, left, top, width = 1) {
            let _switch = new Gtk.Switch();
            let _ModeHbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0);
            if (GTK_MAJOR_VERSION >= 4)
                _ModeHbox.append(_switch);
            else
                _ModeHbox.add(_switch);
            this._grid.attach(_ModeHbox, left, top, width, 1);
            this._settings.bind(binding, _switch, 'active', Gio.SettingsBindFlags.DEFAULT);
        }

        _addEntryRow(text, binding, top){
            let _label = new Gtk.Label({
                label: text,
                halign: Gtk.Align.START,
                use_markup: true,
                visible: true
            });
            this._grid.attach(_label, 0, top, 1, 1);
            let _entry = Gtk.Entry.new();
            this._grid.attach(_entry, 1, top, 1, 1);
            this._settings.bind(binding, _entry, 'text', Gio.SettingsBindFlags.DEFAULT);
        }

        _addSwitchRow(text, binding, top) {
            let _label = new Gtk.Label({
                label: text,
                halign: Gtk.Align.START,
                use_markup: true,
                visible: true
            });
            this._grid.attach(_label, 0, top, 1, 1);

            let _switch = new Gtk.Switch();
            let _ModeHbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0);
            if (GTK_MAJOR_VERSION >= 4)
                _ModeHbox.append(_switch);
            else
                _ModeHbox.add(_switch);
            this._grid.attach(_ModeHbox, 1, top, 1, 1);
            this._settings.bind(binding, _switch, 'active', Gio.SettingsBindFlags.DEFAULT);
        }

        _addSpinButtonRow(text, binding, min, max, step, top) {
            let _label = new Gtk.Label({
                label: text,
                halign: Gtk.Align.START,
                use_markup: true,
                visible: true
            });
            this._grid.attach(_label, 0, top, 1, 1);

            let _pollSpinButton = Gtk.SpinButton.new_with_range( min, max, step );
            this._grid.attach(_pollSpinButton, 1, top, 1, 1);
            this._settings.bind(binding, _pollSpinButton, 'value', Gio.SettingsBindFlags.DEFAULT);
        }

        _init() {
            super._init();
            this._settings = ExtensionUtils.getSettings();

            let posTop = 0;
            this._addGrid(10,10);

            this._addLabel(`<b>${Me.metadata.name} Preferences</b>`, 0, posTop, 1);
            this._addLabel(`<b>Version ${Version}</b>`, 1, posTop++, 1);

            this._addSwitchRow('Debug Mode:', 'serverdebug', posTop++);
            this._addEntryRow('Server IP:', 'serverip', posTop++);
            this._addEntryRow('Server MAC:', 'servermac', posTop++);
            this._addSpinButtonRow('Server Poll:', 'serverpoll', 5, 300, 5, posTop++)
        }

    });

function buildPrefsWidget() {
    const prefsWidget = new ServerPrefsWidget();
    // Return our widget which will be added to the window
    return prefsWidget;
}
